#version 430

layout(location = 0) in vec3 position;
layout(location = 1) in vec3 normal;

uniform mat4 allTransformsInOnematrix;
uniform mat4 modelMatrix;
uniform vec3 color;

uniform float ka; //ambient constant 
uniform float kd; //diffuse constant
uniform float ks; //specular constant
uniform int alpha; //shininess constant

//Directional Light
//lights stored Color - Direction
uniform vec3 directionalLights[6];
uniform int directionalLightsCount = 0;

//Pointlight
//lights stored Color - position - attenuation
uniform vec3 pointLights[9];
uniform int pointLightsCount = 0;


struct spotLight {
	vec3 color;
	vec3 position;
	vec3 direction;
	vec3 attenuation;
	float innerAngle;
	float outerAngle;
};
//spotlight
uniform int spotLExists = 0;
uniform spotLight myspotLight;

uniform vec3 cameraposition;

out vec3 finalcolor;

void main(){
	gl_Position = allTransformsInOnematrix * vec4(position, 1.0);
	vec3 newNormal = normalize(vec3(transpose(inverse(modelMatrix)) * vec4(normal, 0)));
	vec3 newPosition = vec3(modelMatrix * vec4(position, 1.0));
	
	vec3 viewerdirection = normalize(cameraposition - newPosition);
	//Ambient
	vec3 ambient = vec3(1 * ka, ka, ka);
	
	vec3 totaldiffusion = vec3(0,0,0);
	vec3 totalspec = vec3(0,0,0);
	
	
	//Point Lights
	for(int i= 0; i < pointLightsCount*3; i+=3){
		
		vec3 pointColor = pointLights[i];
		vec3 lightposition = pointLights[i+1];
		vec3 pointattenuation = pointLights[i + 2];
		float attenuation = 1/(pointattenuation.x + distance(lightposition, newPosition) * pointattenuation.y + pow(distance(lightposition, newPosition), 2) * pointattenuation.z);
		vec3 intensity = pointColor * vec3(attenuation, attenuation, attenuation);
		
		//Diffuse Pointlight
		vec3 lightVec = normalize(lightposition - newPosition);
		float brightness = dot(lightVec, newNormal);
		vec3 pointdiffuse = intensity * kd * brightness;
		totaldiffusion += pointdiffuse;
		
		//Specular Pointlight
		vec3 reflection = normalize(reflect(-lightVec, newNormal));
		vec3 pointspecular = intensity * ks * pow(clamp(dot(reflection, viewerdirection), 0,1),alpha);
		totalspec += pointspecular;
	}
	
	
	//Directional Lights 
	for(int i = 0; i < directionalLightsCount*2; i+=2){
		vec3 dirColor = directionalLights[i];
		vec3 dirDirection = normalize(directionalLights[i+1]);
		
		//Diffuse Directional Light
		float brightness = clamp(dot(-dirDirection, newNormal), 0,1);
		vec3 dirdiffuse = dirColor * kd * brightness;
		totaldiffusion += dirdiffuse;
	
		//Specular Directional Light
		vec3 reflection = normalize(reflect(dirDirection, newNormal));
		vec3 dirspecular = dirColor * ks * pow(clamp(dot(reflection, viewerdirection),0,1),alpha);
		totalspec += dirspecular;
	}
	
	//Spotlight
	if(spotLExists != 0){
		vec3 spotdirection = -normalize(myspotLight.direction);
		
		
		//Diffuse Spotlight
		vec3 lightVec = normalize(myspotLight.position - newPosition);
		float angle = dot(spotdirection, lightVec);
		if(angle > myspotLight.outerAngle){
			float brightness = dot(lightVec, newNormal);
			float attenuation = 1/(myspotLight.attenuation.x + distance(myspotLight.position, newPosition) * myspotLight.attenuation.y + pow(distance(myspotLight.position, newPosition), 2) * myspotLight.attenuation.z);
			vec3 intensity = myspotLight.color * vec3(attenuation, attenuation, attenuation);
			
			if(angle >= myspotLight.innerAngle){
				//nothing changes
			}else{
				float angleCutoff = myspotLight.innerAngle - myspotLight.outerAngle;
				float inbetween = angle - myspotLight.outerAngle;
				inbetween = inbetween/ angleCutoff;
				intensity = intensity * vec3(inbetween, inbetween, inbetween);
			}
			
			vec3 spotdiffuse = intensity * kd * brightness;
			totaldiffusion += spotdiffuse;
			
			//Specular Spotlight
			vec3 reflection = normalize(reflect(-lightVec, newNormal));
			vec3 spotspecular = intensity * ks * pow(clamp(dot(reflection, viewerdirection), 0,1),alpha);
			totalspec += spotspecular;
		}
	}
	
	finalcolor = color*(ambient + clamp(totaldiffusion, 0, 1)) + clamp(totalspec, 0, 1);
}