#version 430

in vec3 newNormal;
in vec3 newPosition;
in vec2 uv_out;

out vec4 gl_FragColor; 

uniform vec3 color;
uniform float ka; //ambient constant 
uniform float kd; //diffuse constant
uniform float ks; //specular constant
uniform int alpha; //shininess constant

layout (location = 3) uniform sampler2D diffuseTex;
//uniform sampler2D specularMap;
uniform int specMapExists = 0; 
layout (location = 4) uniform samplerCube environmentMap;
uniform float reflectiveness = 0.0;

uniform vec3 cameraposition;

//Pointlight
//lights stored Color - position - attenuation
uniform vec3 pointLights[9];
uniform int pointLightsCount = 0;

//Directional Light
//lights stored Color - Direction
uniform vec3 directionalLights[6];
uniform int directionalLightsCount = 0;

void main(){
	
	vec3 newnormNormal = normalize(newNormal);
	
	//Ambient
	vec3 ambient = vec3(1 * ka, ka, ka);
	
	vec3 totaldiffusion = vec3(0,0,0);
	vec3 totalspec = vec3(0,0,0);
	
	vec3 viewerdirection = normalize(cameraposition - newPosition);
	
	//Point Lights
	for(int i= 0; i < pointLightsCount*3; i+=3){
		
		vec3 pointColor = pointLights[i];
		vec3 lightposition = pointLights[i+1];
		vec3 pointattenuation = pointLights[i + 2];
		float attenuation = 1/(pointattenuation.x + distance(lightposition, newPosition) * pointattenuation.y + pow(distance(lightposition, newPosition), 2) * pointattenuation.z);
		vec3 intensity = pointColor * vec3(attenuation, attenuation, attenuation);
		
		//Diffuse Pointlight
		vec3 lightVec = normalize(lightposition - newPosition);
		float brightness = dot(lightVec, newnormNormal);
		vec3 pointdiffuse = intensity * kd * brightness;
		totaldiffusion += pointdiffuse;
		
		//Specular Pointlight
		vec3 reflection = normalize(reflect(-lightVec, newnormNormal));
		vec3 pointspecular = intensity * ks * pow(clamp(dot(reflection, viewerdirection), 0,1),alpha);
		totalspec += pointspecular;
	}
	
	//Directional Lights 
	for(int i = 0; i < directionalLightsCount*2; i+=2){
		vec3 dirColor = directionalLights[i];
		vec3 dirDirection = normalize(directionalLights[i+1]);
		
		//Diffuse Directional Light
		float brightness = clamp(dot(-dirDirection, newnormNormal), 0,1);
		vec3 dirdiffuse = dirColor * kd * brightness;
		totaldiffusion += dirdiffuse;
	
		//Specular Directional Light
		vec3 reflection = normalize(reflect(dirDirection, newnormNormal));
		vec3 dirspecular = dirColor * ks * pow(clamp(dot(reflection, viewerdirection),0,1),alpha);
		totalspec += dirspecular;
	}
	
	//Environment mapping
	vec3 eyereflection = reflect(-viewerdirection,newNormal);
	vec4 enviroColor = texture(environmentMap, eyereflection);
	float totalEreflection = reflectiveness;
	
	if(specMapExists > 0){
		//vec4 specValue = texture(specularMap, uv_out);
		//totalspec *= vec3(specValue);
		//totalEreflection *= specValue.x;
	}
	
	
	vec3 finalcolor = vec3(texture(diffuseTex, uv_out))* (ambient + clamp(totaldiffusion, 0, 1)) ;
	
	gl_FragColor = mix(vec4(finalcolor, 1.0), enviroColor, totalEreflection) + clamp(totalspec.x, 0, 1);
	
}