#version 430

in vec3 newNormal;
in vec3 newPosition;
in vec2 uv_out;

out vec4 gl_FragColor; 

uniform vec3 color;
uniform float overallBrightness = 1.0f;
uniform float ka; //ambient constant 
uniform float kd; //diffuse constant
uniform float ks; //specular constant
uniform int alpha; //shininess constant

layout (location = 3) uniform sampler2D diffuseTex;
//stuff for specular and environment mapping... 
//uniform sampler2D specularMap;
uniform int specMapExists = 0; 

uniform vec3 cameraposition;

//Pointlight
//lights stored Color - position - attenuation
uniform vec3 pointLights[9];
uniform int pointLightsCount = 0;

//Directional Light
//lights stored Color - Direction
uniform vec3 directionalLights[6];
uniform int directionalLightsCount = 0;

//spotlight
uniform int spotLExists = 0; //spotlight count
uniform vec3 spotlights[25]; //color - position - direction - attenuation - innerAngle/outerAngle/padding

float minimalAmbiance = 0.3;
float levels = 2.0;
float specThreshold = 0.7;
void main(){
	
	vec3 newnormNormal = normalize(newNormal);
	
	//Ambient
	vec3 ambient = vec3( ka, ka, ka) + minimalAmbiance;
	
	vec3 totaldiffusion = vec3(0,0,0);
	vec3 totalspec = vec3(0,0,0);
	
	vec3 viewerdirection = normalize(cameraposition - newPosition);
	
	//Point Lights
	for(int i= 0; i < pointLightsCount*3; i+=3){
		
		vec3 pointColor = pointLights[i];
		vec3 lightposition = pointLights[i+1];
		vec3 pointattenuation = pointLights[i + 2];
		float attenuation = 1/(pointattenuation.x + distance(lightposition, newPosition) * pointattenuation.y + pow(distance(lightposition, newPosition), 2) * pointattenuation.z);
		vec3 intensity = pointColor * vec3(attenuation, attenuation, attenuation);
		
		//Diffuse Pointlight
		vec3 lightVec = normalize(lightposition - newPosition);
		float brightness = dot(lightVec, newnormNormal);
		float level = floor(brightness * levels);
		brightness = level / levels;
		vec3 pointdiffuse = intensity * brightness;
		totaldiffusion += pointdiffuse;
		
		//Specular Pointlight
		vec3 reflection = normalize(reflect(-lightVec, newnormNormal));
		float factor = pow(clamp(dot(reflection, viewerdirection),0,1),alpha);
		//factor = floor(factor * levels) / levels; 
		factor = factor > specThreshold ? 1 : 0;
		vec3 pointspecular = intensity  * factor;
		totalspec += pointspecular;
	}
	
	//Directional Lights 
	for(int i = 0; i < directionalLightsCount*2; i+=2){
		vec3 dirColor = directionalLights[i];
		vec3 dirDirection = normalize(directionalLights[i+1]);
		
		//Diffuse Directional Light
		float brightness = clamp(dot(-dirDirection, newnormNormal), 0,1);
		brightness = floor(brightness * levels)/ levels;
		vec3 dirdiffuse = dirColor * brightness;
		totaldiffusion += dirdiffuse;
	
		//Specular Directional Light
		vec3 reflection = normalize(reflect(dirDirection, newnormNormal));
		float factor = pow(clamp(dot(reflection, viewerdirection),0,1),alpha);
		//factor = floor(factor * levels) / levels; 
		factor = factor > specThreshold ? 1 : 0;
		vec3 dirspecular = dirColor  * factor;
		totalspec += dirspecular;
	}
	for(int i= 0; i < spotLExists * 5; i+=5){
		//color - position - direction - attenuation - innerAngle/outerAngle/padding
		vec3 spotcolor = spotlights[i];
		vec3 position = spotlights[i+1];
		vec3 direction = spotlights[i+2];
		vec3 attenuation = spotlights[i+3];
		float innerAngle = spotlights[i+4].x;
		float outerAngle = spotlights[i+4].y;
		
		vec3 spotdirection = -normalize(direction);
		
		
		//Diffuse Spotlight
		vec3 lightVec = normalize(position - newPosition);
		float angle = dot(spotdirection, lightVec);
		if( angle > outerAngle){
			//float cartoonbrightness = 1.0;
			float brightness = dot(lightVec, newnormNormal);
			brightness = ceil(brightness * levels)/ levels;
			float attenuation = 1/(attenuation.x + distance(position, newPosition) * attenuation.y + pow(distance(position, newPosition), 2) * attenuation.z);
			vec3 intensity = spotcolor * vec3(attenuation, attenuation, attenuation);
			
			if(angle >= innerAngle){ //is in inner cone
				//intensity already correct
			}else{ //in outer cone -> linearer abfall
				//float angleCutoff =innerAngle - outerAngle;
				//float inbetween = angle - outerAngle;
				//inbetween = inbetween/ angleCutoff;
				brightness = brightness * 0.5;
				//cartoonbrightness = 0.5;
			}
			

			vec3 spotdiffuse =intensity * brightness;
			totaldiffusion += spotdiffuse;
			
			//Specular Spotlight
			vec3 reflection = normalize(reflect(-lightVec, newnormNormal));
			float factor = pow(clamp(dot(reflection, viewerdirection),0,1),alpha);
			factor = factor > specThreshold ? 1 : 0;
			vec3 spotspecular = intensity  * factor;
			totalspec += spotspecular;
		}
	}
	
	vec3 finalcolor = vec3(texture(diffuseTex, uv_out))* (ambient + clamp(totaldiffusion * kd, 0, 1)) + clamp(totalspec * ks, 0, 1);
	gl_FragColor = vec4(finalcolor, 1.0)* overallBrightness;
}